package xyz.etztech.minecraftmanager;

import com.google.gson.JsonArray;
import net.md_5.bungee.api.chat.*;
import org.apache.commons.lang.StringUtils;
import org.bukkit.Bukkit;
import org.bukkit.ChatColor;
import org.bukkit.entity.Player;
import xyz.etztech.core.web.CoreWeb;
import xyz.etztech.minecraftmanager.objects.ModelResponse;

import javax.xml.soap.Text;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class MCMUtil {
    private static FileWriter globalFile;
    public static PrintWriter globalLog;
    private static FileWriter staffFile;
    public static PrintWriter staffLog;

    public static void init() {
        MinecraftManager plugin = (MinecraftManager) Bukkit.getPluginManager().getPlugin("MinecraftManager");
        String pluginPath = plugin.getDataFolder().getAbsolutePath();
        String globalPath = pluginPath + "/logs/global.log";
        try {
            globalFile = new FileWriter(globalPath, true);
            globalLog = new PrintWriter(globalFile);
        } catch (Exception e) {
            plugin.log("Could not connect to global log (" + globalPath + ").");
        }
        if (plugin.getConfig().getBoolean("staff-chat.enabled")) {
            String staffPath = pluginPath + "/logs/staff.log";
            try {
                staffFile = new FileWriter(staffPath, true);
                staffLog = new PrintWriter(staffFile);
            } catch (Exception e) {
                plugin.log("Could not connect to staff log (" + staffPath + ").");
            }
        }
    }

    public static void staffMessage(String message) {
        staffMessage(message, false);
    }
    public static void staffMessage(String message, boolean mcm) {
        log(staffLog, message);
        String tag = mcm ? "MCM " : "";
        HoverEvent hover = new HoverEvent(HoverEvent.Action.SHOW_TEXT, new ComponentBuilder(tag + "Staff Chat").create());
        TextComponent text = new TextComponent(message);
        text.setHoverEvent(hover);
        for (Player player : Bukkit.getServer().getOnlinePlayers()) {
            if (player.hasPermission("minecraftmanager.staff")) {
                player.spigot().sendMessage(text);
            }
        }
        Bukkit.getConsoleSender().sendMessage(message);
    }

    @Deprecated
    public static void staffMessage(String message, String hoverCommand) {
        log(staffLog, message);
        HoverEvent hover = new HoverEvent(HoverEvent.Action.SHOW_TEXT, new ComponentBuilder("Staff Chat").create());
        TextComponent text = new TextComponent(message);
        text.setHoverEvent(hover);
        if (StringUtils.isNotEmpty(hoverCommand)) {
            text.setClickEvent(new ClickEvent(ClickEvent.Action.RUN_COMMAND, hoverCommand));
        }
        for (Player player : Bukkit.getServer().getOnlinePlayers()) {
            if (player.hasPermission("minecraftmanager.staff")) {
                player.spigot().sendMessage(text);
            }
        }
        Bukkit.getConsoleSender().sendMessage(message);
    }

    public static void staffMessage(TextComponent message) {
        log(staffLog, message.toPlainText());
        for (Player player : Bukkit.getServer().getOnlinePlayers()) {
            if (player.hasPermission("minecraftmanager.staff")) {
                player.spigot().sendMessage(message);
            }
        }
        Bukkit.getConsoleSender().sendMessage(message.toPlainText());
    }

    public static void globalMessage(String message) {
        log(globalLog, message);
        for (Player player : Bukkit.getServer().getOnlinePlayers()) {
            player.sendMessage(message);
        }
    }

    public static void globalMessage(TextComponent message) {
        log(globalLog, message.toPlainText());
        for (Player player : Bukkit.getServer().getOnlinePlayers()) {
            player.spigot().sendMessage(message);
        }
    }

    public static String getUUID(String playerName) {
        String uuid = null;
        try {
            uuid = Bukkit.getPlayer(playerName).getUniqueId().toString();
        } catch(Exception ex) {
            try {
                Map<String, String> filters = MCMAPI.setup();
                filters.put("username__iexact", playerName);
                ModelResponse response = new ModelResponse(CoreWeb.HTTP(MCMAPI.getModelUrl("player"), CoreWeb.HttpMethod.GET,
                        filters));
                JsonArray players = response.getResults();
                if (players.size() == 1) {
                    uuid = players.get(0).getAsJsonObject().get("uuid").getAsString();
                }
            } catch (Exception ignored) {}
        }
        return uuid;
    }

    public static void log(PrintWriter logger, String message) {
        try {
            //[4/21/17 7:12 PM]
            //Bukkit.getServer().getConsoleSender().sendMessage(message);
            message = ChatColor.stripColor(message);
            Date date = new Date();
            SimpleDateFormat sdf = new SimpleDateFormat("M/d/yy hh:mm a");
            String text = "[" + sdf.format(date) + "] " + message;
            logger.append(text + "\n");
            logger.flush();
        } catch (Exception ex) {
            Bukkit.getServer().getConsoleSender().sendMessage("Could not log message! Check your log files!");
        }
    }
}
